﻿Imports System.Net
Imports System.Xml

Public Enum GoogleGeocodeReturnCode
    Ok = 0
    NoResult
    QueryLimitExceeded
    RequestDenied
    InvalidRequest
End Enum

Public Class GoogleGeocodeException : Inherits Exception

    Private _googleReturnCode As GoogleGeocodeReturnCode
    Public Property GoogleReturnCode() As GoogleGeocodeReturnCode
        Get
            Return _googleReturnCode
        End Get
        Set(ByVal value As GoogleGeocodeReturnCode)
            _googleReturnCode = value
        End Set
    End Property

    Public Sub New()
        MyBase.New()
    End Sub

    Public Sub New(ByVal message As String)
        MyBase.New(message)
    End Sub

    Public Sub New(ByVal message As String, ByVal googleReturnCode As GoogleGeocodeReturnCode)
        MyBase.New(message)
        _googleReturnCode = googleReturnCode
    End Sub
End Class

Public Class GeoCoordinate

#Region " Public Properties"
    Private _latitude As Decimal
    Public Property Latitude() As Decimal
        Get
            Return _latitude
        End Get
        Set(ByVal value As Decimal)
            _latitude = value
        End Set
    End Property

    Private _longitude As Decimal
    Public Property Longitude() As Decimal
        Get
            Return _longitude
        End Get
        Set(ByVal value As Decimal)
            _longitude = value
        End Set
    End Property
#End Region

    Public Sub New(ByVal latitude As Decimal, ByVal longitude As Decimal)
        _latitude = latitude
        _longitude = longitude
    End Sub

End Class

Public Class GoogleGeocode
    Public Shared Function GetCoordinates(ByVal street As String, ByVal city As String, ByVal state As String) As GeoCoordinate
        Dim address As String = street & ", " & city & ", " & state
        Dim wc As New WebClient()
        Dim apiAddr As Uri = GetGeocodeUrl(address)

        Dim geocodeInfo As String = wc.DownloadString(apiAddr)
        Dim coords As GeoCoordinate = Nothing

        If geocodeInfo.Length > 0 Then
            Dim geocodeXml As New XmlDocument
            geocodeXml.LoadXml(geocodeInfo)

            Dim statusNode As XmlNode = geocodeXml.SelectSingleNode("/GeocodeResponse/status")
            If statusNode IsNot Nothing Then
                Select Case statusNode.InnerText
                    Case "OK"
                        Dim locationNode As XmlNode = geocodeXml.SelectSingleNode("/GeocodeResponse/result/geometry/location")
                        If locationNode IsNot Nothing Then
                            Dim lat, lon As Decimal
                            If locationNode.SelectSingleNode("lat") IsNot Nothing AndAlso _
                               Decimal.TryParse(locationNode.SelectSingleNode("lat").InnerText, lat) AndAlso _
                               locationNode.SelectSingleNode("lng") IsNot Nothing AndAlso _
                               Decimal.TryParse(locationNode.SelectSingleNode("lng").InnerText, lon) Then
                                coords = New GeoCoordinate(lat, lon)
                            End If
                        End If
                    Case "ZERO_RESULTS"
                        'do nothing, return empty coords
                    Case "OVER_QUERY_LIMIT"
                        Throw New GoogleGeocodeException("Geocode query limit exceeded.", GoogleGeocodeReturnCode.QueryLimitExceeded)
                    Case "REQUEST_DENIED"
                        Throw New GoogleGeocodeException("Geocode request denied.", GoogleGeocodeReturnCode.RequestDenied)
                    Case "INVALID_REQUEST"
                        Throw New GoogleGeocodeException("Invalid address specified.", GoogleGeocodeReturnCode.InvalidRequest)
                End Select
            End If

            'Dim returnCode As GoogleGeocodeReturnCode = GoogleGeocodeReturnCode.Invalid
            'If Integer.TryParse(geocodeInfo(0), returnCode) Then
            '    Select Case returnCode
            '        Case GoogleGeocodeReturnCode.G_GEO_SUCCESS
            '            coords = New Coordinate(Convert.ToDecimal(geocodeInfo(2)), Convert.ToDecimal(geocodeInfo(3)))
            '        Case GoogleGeocodeReturnCode.G_GEO_SERVER_ERROR
            '            Throw New GoogleGeocodeException("Server error while retrieving geocoded address.", GoogleGeocodeReturnCode.G_GEO_SERVER_ERROR)
            '        Case GoogleGeocodeReturnCode.G_GEO_MISSING_QUERY
            '            Throw New GoogleGeocodeException("Query information missing while retrieving geocoded address.", GoogleGeocodeReturnCode.G_GEO_MISSING_QUERY)
            '        Case GoogleGeocodeReturnCode.G_GEO_UNKNOWN_ADDRESS
            '            Throw New GoogleGeocodeException("Unknown address.", GoogleGeocodeReturnCode.G_GEO_UNKNOWN_ADDRESS)
            '        Case GoogleGeocodeReturnCode.G_GEO_UNAVAILABLE_ADDRESS
            '            Throw New GoogleGeocodeException("Unavailable address.", GoogleGeocodeReturnCode.G_GEO_UNAVAILABLE_ADDRESS)
            '        Case GoogleGeocodeReturnCode.G_GEO_BAD_KEY
            '            Throw New GoogleGeocodeException("Invalid map key.", GoogleGeocodeReturnCode.G_GEO_BAD_KEY)
            '        Case GoogleGeocodeReturnCode.G_GEO_TOO_MANY_QUERIES
            '            Throw New GoogleGeocodeException("Geocode query limit exceeded.", GoogleGeocodeReturnCode.G_GEO_TOO_MANY_QUERIES)
            '    End Select
            'End If
        Else
            Throw New Exception("Unexpected error while geocoding address.")
        End If

        Return coords
    End Function

    Private Shared Function GetGeocodeUrl(ByVal address As String) As Uri
        Dim googleUrl As String = "http://maps.google.com/maps/api/geocode/xml?"
        If ConfigurationManager.AppSettings("googleGeocoderUrl") IsNot Nothing Then
            googleUrl = ConfigurationManager.AppSettings("googleGeocoderUrl")
        End If

        Dim formattedAddress As String = address.Replace("  ", " ")
        formattedAddress = formattedAddress.Replace(" ", "+")

        Return New Uri(googleUrl & "address=" & formattedAddress & "&sensor=false")
    End Function

End Class
